/************************************************************************/
/*				libtabl.c				*/
/*			 Code to handle CTDLTABL.SYS			*/
/************************************************************************/

/************************************************************************/
/*				history 				*/
/*									*/
/* 85Nov15 HAW	Created.						*/
/************************************************************************/

#include "ctdl.h"

/************************************************************************/
/*				Contents				*/
/*									*/
/*	readSysTab()		restores system state from ctdltabl.sys */
/*	writeSysTab()		saves state of system in CTDLTABL.SYS	*/
/************************************************************************/

struct config	 cfg;			/* A buncha variables		*/
struct lTable	 *logTab;		/* RAM index of pippuls 	*/
struct netTable  *netTab;		/* RAM index of nodes		*/
struct rTable	 roomTab[MAXROOMS];	/* RAM index of rooms		*/

FILE *fopen();
char *malloc();

/************************************************************************/
/*	readSysTab() restores state of system from CTDLTABL.SYS 	*/
/*	returns:	TRUE on success, else FALSE			*/
/*	  destroys CTDLTABL.TAB after read, to prevent erroneous re-use */
/*	  in the event of a crash.					*/
/*									*/
/*	MS-DOS fun: Here's the map --					*/
/*	Word 1 == sizeof cfg						*/
/*	Word 2 == sizeof logTab 					*/
/*	Word 3 == sizeof roomTab					*/
/*	Word 4 -- thru x == cfg contents				*/
/*	x -- y == logTab						*/
/*	y -- z == roomTab						*/
/*	z -- a == netTab						*/
/*	EOF								*/
/************************************************************************/
readSysTab(kill)
char kill;
{
    unsigned int  getw();
    unsigned char *c;
    char	  eofflag = FALSE;
    FILE	  *fd;
    unsigned int  size1, size2, size3;

    if ((fd = fopen("ctdlTabl.sys", "rb")) == NULL) {
	printf("?no ctdlTabl.sys!");	/* Tsk, tsk! */
	return(FALSE);
    }

    size1 = getw(fd);
    eofflag = ferror(fd);
    size2 = getw(fd);
    eofflag = (eofflag) ? eofflag : ferror(fd);
    size3 = getw(fd);
    eofflag = (eofflag) ? eofflag : ferror(fd);
    if (eofflag) {
	printf("?error reading ctdlTabl.sys!");
	return(FALSE);
    }
    if (size1 != sizeof cfg    ||
	size3 != sizeof roomTab) {
	printf("?old ctdlTabl.sys!");
	return(FALSE);
    }

    logTab = (struct lTable *) malloc(size2);

    c	= &cfg.firstExtern;
    while (c < &cfg.lastExtern)
	if ((*c++ = getc(fd)) == ERROR) {
	    printf("?error reading ctdlTabl.sys!");
	    return FALSE;
	}

    if (size2 != sizeof(*logTab) * cfg.MAXLOGTAB) {
	printf("?old ctdlTabl.sys!");
	return(FALSE);
    }

    c	= &logTab[0];
    while (c < &logTab[cfg.MAXLOGTAB])
	if ((*c++ = getc(fd)) == ERROR) {
	    printf("?error reading ctdlTabl.sys!");
	    return FALSE;
	}

    c	= &roomTab[0];
    while (c < &roomTab[MAXROOMS])
	if ((*c++ = getc(fd)) == ERROR) {
	    printf("?error reading ctdlTabl.sys!");
	    return FALSE;
	}

    netTab = (struct netTable *) malloc(sizeof (*netTab) * cfg.netSize);
    c = &netTab[0];
    while (c < &netTab[cfg.netSize])
	if ((*c++ = getc(fd)) == ERROR) {
	    printf("?error reading ctdlTabl.sys!");
	    return FALSE;
	}

    if (kill) unlink("ctdlTabl.sys");
    return(TRUE);
}

/************************************************************************/
/*	writeSysTab() saves state of system in CTDLTABL.SYS		*/
/*	returns:	TRUE on success, else ERROR			*/
/*	See readSysTab() to see what the CTDLTABL.SYS map looks like.	*/
/************************************************************************/
writeSysTab()
{
    unsigned char *c;
    FILE	  *fd;

    if ((fd = fopen("ctdlTabl.sys", "wb")) == NULL) {
	printf("?can't make ctdlTabl.sys");
	return(ERROR);
    }

    /* Write out some key stuff so we can detect bizarreness: */
    putw(sizeof cfg    , fd);
    putw(sizeof(*logTab) * cfg.MAXLOGTAB, fd);
    putw(sizeof roomTab, fd);
    c	= &cfg.firstExtern;
    while (c < &cfg.lastExtern)
	putc(*c++, fd);

    c	= &logTab[0];
    while (c < &logTab[cfg.MAXLOGTAB])
	putc(*c++, fd);

    c	= &roomTab[0];
    while (c < &roomTab[MAXROOMS])
	putc(*c++, fd);

    c	= &netTab[0];
    while (c < &netTab[cfg.netSize])
	putc(*c++, fd);

    fclose(fd);
    return(TRUE);
}
